// Dixit - Copyright (C) 2004-2010 Octavian Procopiuc
// Contact: <oprocopiuc@gmail.com>
// Homepage: <http://dixit.sourceforge.net>
//
// Use of this source code is governed by the GPLv3
// license that can be found in the LICENSE.GPL3 file,
// or at <http://www.gnu.org/licenses/gpl.html>.
//
// FILENAME:   mainwindow.h
// REVISION:   3
// COMMENTARY: Header file for Qt4 MainWindow class
// NAMING:     a_ MainWindow action
//             m_ MainWindow member variable
//             w_ MainWindow member widget

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "depot.h"
#include <QDir>
#include <QHttp>
#include <QHttpResponseHeader>
#include <QListWidget>
#include <QMainWindow>
#include <QTimer>
#include <QtGui>
#include <QUrl>

class QAction;
class QComboBox;
class QDir;
class QHttp;
class QHttpResponseHeader;
class QLabel;
class QLineEdit;
class QListWidget;
class QSplitter;
class QTextBrowser;
class QTimer;
class QUrl;

enum UpdateErrors {
  UPDATE_ERR_NO_ERROR,
  UPDATE_ERR_NETWORK,
  UPDATE_ERR_SERVER,
  UPDATE_ERR_DB_READ,
  UPDATE_ERR_DB_WRITE,
  UPDATE_ERR_CONTENT,
  UPDATE_ERR_STREAM,
  UPDATE_ERR_TIMEOUT,
  UPDATE_ERR_NOT_DONE,
  UPDATE_ERR_ABORTED
};

class MainWindow : public QMainWindow
{
  Q_OBJECT

public:
  MainWindow(void);
  ~MainWindow(void);
  int dbCreateIndex(void);

signals:
  void moreData(const QHttpResponseHeader &);

protected:
  QThread *m_pThread;

private slots:
  void abortUpdate(void);
  void anchorClicked(const QUrl &link);
  void backwardInHistory(void);
  void changeFont(void);
  void checkHttpResponseHeader(const QHttpResponseHeader &rh);
  void closeConnection(bool done);
  void connectionTimeout(void);
  void forwardInHistory(void);
  void getDexonlineData(const QHttpResponseHeader &rh);
  void help(void);
  void makeFontBigger(void);
  void makeFontSmaller(void);
  void print(void);
  void search(void);
  void setDefaultDictionary(int id);
  void showDictionary(int id);
  void update(void);
  void wordClickedInHistoryBox(const QString &word);

private:
#if QT_VERSION < 0x040500
  void    appendToByteArray(QByteArray &ba, const char *buf, uint bufsz);
#endif
  void    clearBrowser(void);
  void    createIndependentActions(void);
  void    createMainWindow(void);
  void    createStatusBar(void);
  void    createToolBar(void);
  void    dbStartIndexing(void);
  void    findDefinitions(const QString &word);
  void    genericUpdate(void);
  void    htmlizeDefinition(QString &html, const char *in);
  void    inflateData(QByteArray &idata, char *zdata, int zdatasz, bool init_only);
  void    insertInSearchBox(const QString &word);
  void    loadSettings(void);
  void    logInfo(const QString msg);
  void    processWord(const QString &word);
  void    saveSettings(void);
  void    setBrowserHtml(const QString &str, bool prs = true);
  void    setFocusToSearchBox(void);
  bool    dbClose(void);
  bool    dbKeyExists(const char *key);
  bool    dbOpenRead(void);
  bool    dbOpenWrite(void);
  bool    dbPut(const char *key, const char *data);
  bool    dbSetup(void);
  int     dbSaveXMLData(const QString &xmlData, UpdateErrors *updateError, bool *abortHttp);
  int     LevenshteinDistance(const QString &source, const QString &target, int maxdist);
  char   *dbGet(const char *key, bool *ok);
  QString dbResults(const QString &word);
  QString formatRoDate(int seconds, bool hour = false);
  QString getDictionaryName(const QString &shortName);
  QString htmlizeBrowserEntry(const QString &dbRec);
  QString loadFile(const QString &fileName);
  QString processCommands(const QString &cmd);

  bool          m_activeConnection;
  bool          m_autoUpdate;
  bool          m_logInfo;
  char          m_dbFileName[128];
  int           m_lastUpdateTime;
  int           m_lastUpdateTimePending;
  int           m_defaultDictionary;
  int           m_newDefCount;
  int           m_newDefCountPending;
  int           m_totalDefinitions;
  UpdateErrors  m_updateError;
  QString       m_strBrowserHtml;
  QString       m_strCurrentWord;
  QStringList   m_lstAllWords;
  QStringList   m_lstBackwardWords;
  QStringList   m_lstForwardWords;
  QStringList   m_lstNewDefinitions;
  QTimer        m_connectionTimer;
  QHttp        *m_http;
  DEPOT        *m_dbHandle;

  QComboBox    *w_dictionaryComboBox;
  QListWidget  *w_historyListBox;
  QLineEdit    *w_searchBox;
  QTextBrowser *w_browser;
  QToolBar     *w_toolBar;

  QAction      *a_next;
  QAction      *a_prev;
};

class dbIndexThread: public QThread
{
  MainWindow *window;
public:
  dbIndexThread(MainWindow *w): QThread(), window(w) { }
  void run() {
    window->dbCreateIndex();
  }
};

#endif  // MAINWINDOW_H

// END OF FILE

